@@ -1,14 +1,14 @@
 package Sidef {
 
-    use 5.014;
-    our $VERSION = '3.00';
+    use utf8;
+    use 5.016;
 
-    our $SPACES      = 0;    # the current number of spaces
-    our $SPACES_INCR = 4;    # the number of spaces incrementor
+    our $VERSION = '3.97.1';
+
+    our $SPACES      = 0;    # the current number of indentation spaces
+    our $SPACES_INCR = 4;    # the number of indentation spaces
 
-    our @NAMESPACES;         # will keep track of declared modules
     our %INCLUDED;           # will keep track of included modules
-
     our %EVALS;              # will contain info required for eval()
 
     use constant {
@@ -28,11 +28,12 @@ package Sidef {
         bless \%opt, $class;
     }
 
+    *call = \&new;
+
     sub parse_code {
         my ($self, $code) = @_;
 
         local %INCLUDED;
-        local @NAMESPACES;
 
         $self->{parser} //= Sidef::Parser->new(
                                                opt         => $self->{opt},
@@ -42,7 +43,6 @@ package Sidef {
                                               );
 
         my $ast = $self->{parser}->parse_script(code => \$code);
-        $self->{namespaces} = \@NAMESPACES;
 
         # Check for optimization
         if (defined(my $level = $self->{opt}{O})) {
@@ -114,27 +114,19 @@ package Sidef {
     sub has_dbm_driver {
         my ($self) = @_;
 
-        if (exists($self->{_db}) or exists($self->{_gdbm})) {
-            return ($self->{_db} || $self->{_gdbm});
+        if (exists $self->{dbm_driver}) {
+            return $self->{dbm_driver};
         }
 
         if (eval { require DB_File; 1 }) {
-            $self->{_db} = 1;
-            return 1;
-        }
-        else {
-            $self->{_db} = 0;
+            return ($self->{dbm_driver} = 'bdbm');
         }
 
         if (eval { require GDBM_File; 1 }) {
-            $self->{_gdbm} = 1;
-            return 1;
-        }
-        else {
-            $self->{_gdbm} = 0;
+            return ($self->{dbm_driver} = 'gdbm');
         }
 
-        return;
+        ($self->{dbm_driver} = undef);
     }
 
     sub _init_db {
@@ -239,8 +231,7 @@ package Sidef {
         if (
             $self->{opt}{s}
             ##and length($$code) > 1024
-            and (defined($self->{dbm_driver})
-                 or $self->has_dbm_driver)
+            and (defined($self->{dbm_driver}) or $self->has_dbm_driver)
           ) {
 
             my $db_dir = ($self->{$lang}{db_dir} //= File::Spec->catdir($self->get_sidef_vdir(), $lang));
@@ -257,12 +248,6 @@ package Sidef {
 
             my $md5 = Digest::MD5::md5_hex(Encode::encode_utf8($code));
 
-            $self->{dbm_driver} //= (
-                                       $self->{_gdbm} ? 'gdbm'
-                                     : $self->{_db}   ? 'bdbm'
-                                     :                  die "unknown DBM driver"
-                                    );
-
             $self->{$lang}{time_db} //= File::Spec->catfile($db_dir, 'Sidef_Time_' . $self->{dbm_driver} . '.db');
             $self->{$lang}{code_db} //= File::Spec->catfile($db_dir, 'Sidef_Code_' . $self->{dbm_driver} . '.db');
 
@@ -308,11 +293,8 @@ package Sidef {
         my $pm     = ($module =~ s{::}{/}gr . '.pm');
 
         require $pm;
-        $self->{$lang}{deparser} = $module->new(
-                                                opt              => $self->{opt},
-                                                namespaces       => $self->{namespaces},
-                                                environment_name => $self->{environment_name} // '',
-                                               );
+        $self->{$lang}{deparser} = $module->new(opt              => $self->{opt},
+                                                environment_name => $self->{environment_name} // '',);
 
         scalar $self->{$lang}{deparser}->deparse($ast);
     }
@@ -325,14 +307,16 @@ package Sidef {
         my ($type) = @_;
 
         if (index($type, 'Sidef::') == 0) {
-            $type = substr($type, rindex($type, '::') + 2);
-        }
-        else {
-            $type =~ s/^(?:_::)?main:://
-              or $type =~ s/^_:://;
+
+            if ($type =~ /::[0-9]+::/) {
+                $type = substr($type, $+[0]);
+            }
+            else {
+                $type = substr($type, rindex($type, '::') + 2);
+            }
         }
 
-        $type =~ s/[0-9]{8}\z//r;
+        $type =~ s/^main:://r;
     }
 
     sub normalize_method {
@@ -358,7 +342,7 @@ package Sidef {
         foreach my $i (0 .. $s_len - 1) {
 
             my $start = List::Util::max(0, $i - $match_distance);
-            my $end = List::Util::min($i + $match_distance + 1, $t_len);
+            my $end   = List::Util::min($i + $match_distance + 1, $t_len);
 
             foreach my $j ($start .. $end - 1) {
                 $t_matches[$j] and next;
@@ -421,6 +405,11 @@ package Sidef {
 
 };
 
+use utf8;
+use 5.016;
+
+our $AUTOLOAD;
+
 #
 ## UNIVERSAL methods
 #
@@ -430,7 +419,9 @@ package Sidef {
       ? $_[0]->{module}
       : $_[0];
 };
+
 *UNIVERSAL::DESTROY = sub { };
+
 *UNIVERSAL::AUTOLOAD = sub {
     my ($self, @args) = @_;
 
@@ -458,18 +449,18 @@ package Sidef {
     my $from   = Sidef::normalize_method($caller[3]);
     $from = $from eq '.' ? 'main()' : "$from()";
 
-    my $table = \%{$self . '::'};
+    my $table   = do { no strict 'refs'; \%{$self . '::'} };
     my @methods = grep { !ref($table->{$_}) and defined(&{$table->{$_}}) } keys(%$table);
 
     my $method = Sidef::normalize_method($AUTOLOAD);
-    my $name = substr($method, rindex($method, '.') + 1);
+    my $name   = substr($method, rindex($method, '.') + 1);
 
     my @candidates = Sidef::best_matches($name, \@methods);
 
     die(  "[AUTOLOAD] Undefined method `"
         . $method . q{'}
         . " called from $from\n"
-        . (@candidates ? ("[?] Did you mean: " . join("\n" . (' ' x 18), sort @candidates) . "\n") : ''));
+        . (@candidates ? ("[?] Did you mean: " . join("\n" . (' ' x 18), sort(@candidates)) . "\n") : ''));
     return;
 };
 
