@@ -1,7 +1,7 @@
 package Sidef::Math::Math {
 
     use utf8;
-    use 5.014;
+    use 5.016;
 
     use parent qw(
       Sidef::Object::Object
@@ -13,92 +13,100 @@ package Sidef::Math::Math {
         bless {}, __PACKAGE__;
     }
 
+    sub binsplit {
+        my ($self, $block, @list) = @_;
+        Sidef::Types::Number::Number::_binsplit(\@list, $block);
+    }
+
     sub gcd {
         my ($self, @list) = @_;
-
-        my $gcd = $list[0];
-        foreach my $i (1 .. $#list) {
-            last if ($gcd->is_one);
-            $gcd = $gcd->gcd($list[$i]);
-        }
-
-        $gcd;
+        Sidef::Types::Number::Number::gcd(@list);
     }
 
     sub lcm {
         my ($self, @list) = @_;
-
-        my $lcm = $list[0];
-        foreach my $i (1 .. $#list) {
-            $lcm = $lcm->lcm($list[$i]);
-        }
-
-        $lcm;
+        Sidef::Types::Number::Number::lcm(@list);
     }
 
     sub sum {
         my ($self, @list) = @_;
-
-        my $sum = Sidef::Types::Number::Number::ZERO;
-        foreach my $n (@list) {
-            $sum = $sum->add($n);
-        }
-
-        $sum;
+        Sidef::Types::Number::Number::sum(@list);
     }
 
     sub prod {
         my ($self, @list) = @_;
-
-        my $prod = Sidef::Types::Number::Number::ONE;
-        foreach my $n (@list) {
-            $prod = $prod->mul($n);
-        }
-
-        $prod;
+        Sidef::Types::Number::Number::prod(@list);
     }
 
     *product = \&prod;
 
     sub max {
         my ($self, @list) = @_;
-
-        my $max = $list[0];
-        foreach my $i (1 .. $#list) {
-            $max = $max->max($list[$i]);
-        }
-
-        $max;
+        Sidef::Types::Number::Number::max(@list);
     }
 
     sub min {
         my ($self, @list) = @_;
-
-        my $min = $list[0];
-        foreach my $i (1 .. $#list) {
-            $min = $min->min($list[$i]);
-        }
-
-        $min;
+        Sidef::Types::Number::Number::min(@list);
     }
 
-    sub avg {
+    sub arithmetic_mean {
         my ($self, @list) = @_;
 
-        my $sum = Sidef::Types::Number::Number::ZERO;
-        foreach my $n (@list) {
-            $sum = $sum->add($n);
-        }
+        my $sum = Sidef::Types::Number::Number::sum(@list);
+        my $n   = Sidef::Types::Number::Number->_set_uint(scalar(@list));
 
-        my $n = Sidef::Types::Number::Number->new(scalar(@list));
         $sum->div($n);
     }
 
+    *avg = \&arithmetic_mean;
+
+    sub geometric_mean {
+        my ($self, @list) = @_;
+
+        my $prod = Sidef::Types::Number::Number::prod(@list);
+        my $n    = Sidef::Types::Number::Number->_set_uint(scalar(@list));
+
+        $prod->root($n);
+    }
+
+    sub harmonic_mean {
+        my ($self, @list) = @_;
+
+        my $sum = Sidef::Types::Number::Number::sum(map { $_->inv } @list);
+        my $n   = Sidef::Types::Number::Number->_set_uint(scalar(@list));
+
+        $n->div($sum);
+    }
+
+    sub smooth_numbers {
+        my ($self, @primes) = @_;
+
+        my @s = map { [Sidef::Types::Number::Number::ONE] } @primes;
+
+        Sidef::Object::Enumerator->new(
+            Sidef::Types::Block::Block->new(
+                code => sub {
+                    my ($callback) = @_;
+
+                    while (1) {
+                        my $n = Sidef::Types::Number::Number::min(map { $_->[0] } @s);
+
+                        for my $i (0 .. $#primes) {
+                            shift(@{$s[$i]}) if $s[$i][0]->eq($n);
+                            push(@{$s[$i]}, $n->mul($primes[$i]));
+                        }
+
+                        $callback->run($n);
+                    }
+                }
+            )
+        );
+    }
+
     sub chinese {
         my ($self, @arrs) = @_;
 
-        state $x = require Scalar::Util;
-
         my @pairs;
         foreach my $pair (@arrs) {
             if (    Scalar::Util::reftype($pair) eq 'ARRAY'
@@ -114,11 +122,11 @@ package Sidef::Math::Math {
 
         my $res = eval { Math::Prime::Util::GMP::chinese(@pairs) } // return Sidef::Types::Number::Number->nan;
 
-        if ($res <= Sidef::Types::Number::Number::ULONG_MAX and $res >= 0) {
+        if ($res < Sidef::Types::Number::Number::ULONG_MAX and $res >= 0) {
             return Sidef::Types::Number::Number->_set_uint($res);
         }
 
-        Sidef::Types::Number::Number->_set_str($res);
+        Sidef::Types::Number::Number->_set_str('int', $res);
     }
 
     sub range_sum {
@@ -147,28 +155,6 @@ package Sidef::Math::Math {
         state $hundred = Sidef::Types::Number::Number->_set_uint(100);
         ($sum->sub($dist))->div($sum)->mul($hundred);
     }
-
-    #~ our $AUTOLOAD;
-
-    #~ #
-    #~ ## This method is highly deprecated!
-    #~ #
-    #~ sub AUTOLOAD {
-    #~ my ($self, $x, @args) = @_;
-    #~ my ($method) = ($AUTOLOAD =~ /^.*[^:]::(.*)$/);
-
-    #~ my $code =
-    #~ ref($x)
-    #~ ? UNIVERSAL::can($x,                             $method)
-    #~ : UNIVERSAL::can('Sidef::Types::Number::Number', $method);
-
-    #~ defined($code)
-    #~ ? $code->(defined($x) ? ($x, @args) : ())
-    #~ : do {
-    #~ my $arg = join(', ', map { defined($_) ? $_ : 'nil' } (@_ > 1 ? ($x, @args) : ()));
-    #~ die "[ERROR] Undefined method Math.$method($arg)";
-    #~ };
-    #~ }
 }
 
-1
+1;
