@@ -1,6 +1,7 @@
 package Sidef::Object::Enumerator {
 
-    use 5.014;
+    use utf8;
+    use 5.016;
     ##use overload q{""} => \&to_a;
 
     sub new {
@@ -54,17 +55,7 @@ package Sidef::Object::Enumerator {
     sub each {
         my ($self, $block) = @_;
 
-        $self->{block}->run(
-            Sidef::Types::Block::Block->new(
-                code => sub {
-                    @_
-                      ? @_ == 1
-                          ? $block->run($_[0])
-                          : do { $block->run($_) for @_ }
-                      : ();
-                },
-            )
-        );
+        $self->{block}->run($block);
 
         $self;
     }
@@ -84,6 +75,8 @@ package Sidef::Object::Enumerator {
         Sidef::Types::Array::Array->new(\@arr);
     }
 
+    *collect = \&map;
+
     sub grep {
         my ($self, $block) = @_;
 
@@ -113,7 +106,7 @@ package Sidef::Object::Enumerator {
             )
         );
 
-        Sidef::Types::Number::Number->_new_uint($count);
+        Sidef::Types::Number::Number->_set_uint($count);
     }
 
     *len  = \&length;    # alias
