@@ -1,6 +1,6 @@
 package Sidef::Optimizer {
 
-    use 5.014;
+    use 5.016;
     use Scalar::Util qw(refaddr);
 
     use constant {
@@ -9,7 +9,9 @@ package Sidef::Optimizer {
                   REGEX       => 'Sidef::Types::Regex::Regex',
                   BOOL        => 'Sidef::Types::Bool::Bool',
                   ARRAY       => 'Sidef::Types::Array::Array',
-                  COMPLEX     => 'Sidef::Types::Number::Complex',
+                  RANGENUM    => 'Sidef::Types::Range::RangeNumber',
+                  DIR_DT      => 'Sidef::DataTypes::Glob::Dir',
+                  FILE_DT     => 'Sidef::DataTypes::Glob::File',
                   NUMBER_DT   => 'Sidef::DataTypes::Number::Number',
                   STRING_DT   => 'Sidef::DataTypes::String::String',
                   COMPLEX_DT  => 'Sidef::DataTypes::Number::Complex',
@@ -18,6 +20,29 @@ package Sidef::Optimizer {
                   BACKTICK_DT => 'Sidef::DataTypes::Glob::Backtick',
                  };
 
+    my %dt_table = (
+        qw(
+          Sidef::DataTypes::Bool::Bool            Sidef::Types::Bool::Bool
+          Sidef::DataTypes::Array::Array          Sidef::Types::Array::Array
+          Sidef::DataTypes::Array::Pair           Sidef::Types::Array::Pair
+          Sidef::DataTypes::Array::Matrix         Sidef::Types::Array::Matrix
+          Sidef::DataTypes::Hash::Hash            Sidef::Types::Hash::Hash
+          Sidef::DataTypes::Set::Set              Sidef::Types::Set::Set
+          Sidef::DataTypes::Set::Bag              Sidef::Types::Set::Bag
+          Sidef::DataTypes::Regex::Regex          Sidef::Types::Regex::Regex
+          Sidef::DataTypes::String::String        Sidef::Types::String::String
+          Sidef::DataTypes::Number::Number        Sidef::Types::Number::Number
+          Sidef::DataTypes::Number::Complex       Sidef::Types::Number::Complex
+          Sidef::DataTypes::Range::RangeNumber    Sidef::Types::Range::RangeNumber
+          Sidef::DataTypes::Range::RangeString    Sidef::Types::Range::RangeString
+          Sidef::DataTypes::Glob::Backtick        Sidef::Types::Glob::Backtick
+          Sidef::DataTypes::Glob::Socket          Sidef::Types::Glob::Socket
+          Sidef::DataTypes::Glob::Pipe            Sidef::Types::Glob::Pipe
+          Sidef::DataTypes::Glob::Dir             Sidef::Types::Glob::Dir
+          Sidef::DataTypes::Glob::File            Sidef::Types::Glob::File
+          )
+    );
+
     my %cache;
     {
 
@@ -30,53 +55,26 @@ package Sidef::Optimizer {
                     defined(my $method = UNIVERSAL::can($package, $_))
                       or die "[ERROR] Invalid method $package: $_";
                     $method;
-                  }
+                }
             } @names;
         }
 
-        my %table = (
-            qw(
-              Sidef::DataTypes::Bool::Bool            Sidef::Types::Bool::Bool
-              Sidef::DataTypes::Array::Array          Sidef::Types::Array::Array
-              Sidef::DataTypes::Array::Pair           Sidef::Types::Array::Pair
-              Sidef::DataTypes::Array::MultiArray     Sidef::Types::Array::MultiArray
-              Sidef::DataTypes::Hash::Hash            Sidef::Types::Hash::Hash
-              Sidef::DataTypes::Regex::Regex          Sidef::Types::Regex::Regex
-              Sidef::DataTypes::String::String        Sidef::Types::String::String
-              Sidef::DataTypes::Number::Number        Sidef::Types::Number::Number
-              Sidef::DataTypes::Number::Complex       Sidef::Types::Number::Complex
-              Sidef::DataTypes::Range::RangeNumber    Sidef::Types::Range::RangeNumber
-              Sidef::DataTypes::Range::RangeString    Sidef::Types::Range::RangeString
-              Sidef::DataTypes::Glob::Backtick        Sidef::Types::Glob::Backtick
-              Sidef::DataTypes::Glob::Socket          Sidef::Types::Glob::Socket
-              Sidef::DataTypes::Glob::Pipe            Sidef::Types::Glob::Pipe
-              Sidef::DataTypes::Glob::Dir             Sidef::Types::Glob::Dir
-              Sidef::DataTypes::Glob::File            Sidef::Types::Glob::File
-              )
-        );
-
-        my %seen;
-
         sub dtypes {
             my ($type, @names) = @_;
 
-            exists($table{$type}) || die "[ERROR] Non-existent data type: $type";
+            exists($dt_table{$type}) || die "[ERROR] Non-existent data type: $type";
+
+            my $package = $dt_table{$type};
+            my $module  = ($cache{$package} //= (($package =~ s{::}{/}gr) . '.pm'));
 
-            my $package = $table{$type};
-            my $module = ($cache{$package} //= (($package =~ s{::}{/}gr) . '.pm'));
             exists($INC{$module}) || require($module);
 
-            if (not $seen{$type}++) {
-                no strict 'refs';
-                push @{$type . '::' . 'ISA'}, $package;
-            }
-
             map {
                 $cache{$type, $_} //= do {
-                    defined(my $method = UNIVERSAL::can($type, $_))
-                      or die "[ERROR] Invalid method $type: $_";
+                    defined(my $method = UNIVERSAL::can($package, $_))
+                      or die "[ERROR] Invalid method $package: $_";
                     $method;
-                  }
+                }
             } @names;
         }
     }
@@ -273,22 +271,31 @@ package Sidef::Optimizer {
 
             # Number.method(Number)
             (
-             map { [$_, [table(NUMBER, COMPLEX)]] }
+             map { [$_, [table(NUMBER)]] }
                methods(NUMBER, qw(
                    + - / * % %% **
 
-                   lt gt le ge cmp acmp
+                   lt gt le ge cmp
                    eq ne
                    and or xor
 
+                   sigma
+                   usigma
+
+                   prime_sigma
+                   prime_usigma
+
+                   is_power
+
                    complex
                    root iroot
                    log
                    next_pow
                    max min
                    roundf
-                   nok
-                   modinv
+                   binomial
+                   subfactorial
+                   invmod
                    isub
                    iadd
                    imul
@@ -312,9 +319,39 @@ package Sidef::Optimizer {
                    inc dec not
 
                    factorial
+                   subfactorial
+                   superfactorial
+                   hyperfactorial
+                   lnsuperfactorial
+                   lnhyperfactorial
                    sqrt isqrt
                    next_pow2
-                   abs int
+                   abs int rat float complex
+                   norm conj sqr
+
+                   zeta
+                   eta
+                   beta
+                   gamma
+                   Ai
+                   Li
+                   Li2
+                   Ei
+
+                   sigma
+                   sigma0
+
+                   usigma
+                   usigma0
+
+                   prime_sigma
+                   prime_usigma
+
+                   divisors
+                   udivisors
+
+                   factor
+                   factor_exp
 
                    exp
                    exp2
@@ -367,29 +404,38 @@ package Sidef::Optimizer {
                    sign
                    nan
                    chr
-                   pi
-                   ln2
-                   phi
-                   tau
-                   e
-                   Y
-                   G
+
+                   i pi tau
+                   e ln2 phi
+                   EulerGamma
+                   CatalanG
 
                    is_zero
                    is_one
                    is_nan
-                   is_positive
-                   is_negative
+                   is_pos
+                   is_neg
                    is_even
                    is_odd
                    is_inf
                    is_ninf
                    is_int
 
+                   is_prime
+                   is_square
+                   is_power
+
+                   sgn
                    ceil
                    floor
                    length
 
+                   ilog
+                   ilog2
+                   ilog10
+
+                   isqrt
+
                    numerator
                    denominator
 
@@ -399,9 +445,7 @@ package Sidef::Optimizer {
                    as_oct
                    as_hex
                    as_rat
-
-                   rat
-                   complex i
+                   as_frac
 
                    dump
                    commify
@@ -413,7 +457,7 @@ package Sidef::Optimizer {
             (
              map { [$_, [table(NUMBER), table(NUMBER)]] }
                methods(NUMBER, qw(
-                   modpow
+                   powmod
                    range
                    )
                )
@@ -442,6 +486,7 @@ package Sidef::Optimizer {
                    first
                    last
 
+                   freq
                    len end
                    is_empty
                    min max
@@ -449,6 +494,12 @@ package Sidef::Optimizer {
                    sum
                    prod
 
+                   zip
+                   cartesian
+                   permutations
+                   derangements
+                   circular_permutations
+
                    sort
                    reverse
 
@@ -471,7 +522,15 @@ package Sidef::Optimizer {
                    exists
                    defined
                    contains
-                   contains_type
+
+                   subsets
+                   partitions
+                   variations
+                   variations_with_repetition
+                   combinations
+                   combinations_with_repetition
+
+                   nth_permutation
 
                    div
                    mul
@@ -501,7 +560,6 @@ package Sidef::Optimizer {
                    concat
 
                    eq ne
-                   mzip
                    contains_type
                    contains_any
                    contains_all
@@ -528,87 +586,25 @@ package Sidef::Optimizer {
             (map { [$_, [table('')]] } methods(ARRAY, qw(reduce_operator))),
                              ),
 
-        (COMPLEX) => build_tree(
+        (RANGENUM) => build_tree(
 
-            # Complex.method(Complex|Number)
+            # RangeNum.method(Number)
             (
-             map { [$_, [table(COMPLEX, NUMBER)]] }
-               methods(COMPLEX, qw(
-                   cmp gt lt ge le eq ne
-                   roundf
-
-                   mul
-                   div
-                   add
-                   sub
-                   log
-                   pow
-
-                   atan2
+             map { [$_, [table(NUMBER)]] }
+               methods(RANGENUM, qw(
+                   from to by
+                   sum add sub mul div
                    )
                )
             ),
 
-            # Complex.method()
+            # RangeNum.method()
             (
              map { [$_, []] }
-               methods(COMPLEX, qw(
-
-                   inc
-                   dec
-                   abs
-
-                   exp
-                   exp2
-                   exp10
-
-                   log
-                   log2
-                   log10
-                   sqrt
-
-                   cos
-                   sin
-                   tan
-                   csc
-                   sec
-                   cot
-                   asin
-                   acos
-                   atan
-                   acsc
-                   asec
-                   acot
-                   sinh
-                   cosh
-                   tanh
-                   csch
-                   sech
-                   coth
-                   asinh
-                   acosh
-                   atanh
-                   acsch
-                   asech
-                   acoth
-
-                   neg
-                   not
-
-                   real
-                   imaginary
-
-                   is_zero
-                   is_one
-                   is_nan
-                   is_real
-                   is_inf
-                   is_int
-
-                   ceil
-                   floor
-
-                   dump
+               methods(RANGENUM, qw(
+                   first last reverse
+                   min max step
+                   sum length
                    )
                )
             ),
@@ -623,9 +619,9 @@ package Sidef::Optimizer {
                    pi
                    tau
                    ln2
-                   Y
-                   G
-                   e
+                   EulerGamma
+                   CatalanG
+                   e i
                    phi
                    nan
                    inf
@@ -686,6 +682,30 @@ package Sidef::Optimizer {
             ),
         ),
 
+        (FILE_DT) => build_tree(
+
+            # File.method(STRING)
+            (
+             map { [$_, [table(STRING)]] }
+               dtypes(FILE_DT, qw(
+                   new
+                   )
+               )
+            ),
+        ),
+
+        (DIR_DT) => build_tree(
+
+            # Dir.method(STRING)
+            (
+             map { [$_, [table(STRING)]] }
+               dtypes(DIR_DT, qw(
+                   new
+                   )
+               )
+            ),
+        ),
+
         (RANGENUM_DT) => build_tree(
 
             # RangeNum.method(NUMBER, NUMBER)
@@ -697,7 +717,7 @@ package Sidef::Optimizer {
                )
             ),
 
-            # RangeNum.method(NUMBER, NUMBER)
+            # RangeNum.method(NUMBER, NUMBER, NUMBER)
             (
              map { [$_, [table(NUMBER), table(NUMBER), table(NUMBER)]] }
                dtypes(RANGENUM_DT, qw(
@@ -733,31 +753,7 @@ package Sidef::Optimizer {
 
             # Complex.method(NUMBER|STRING, NUMBER|STRING)
             (
-             map { [$_, [table(STRING), table(STRING)]] }
-               dtypes(COMPLEX_DT, qw(
-                   new
-                   )
-               )
-            ),
-
-            (
-             map { [$_, [table(STRING), table(NUMBER)]] }
-               dtypes(COMPLEX_DT, qw(
-                   new
-                   )
-               )
-            ),
-
-            (
-             map { [$_, [table(NUMBER), table(STRING)]] }
-               dtypes(COMPLEX_DT, qw(
-                   new
-                   )
-               )
-            ),
-
-            (
-             map { [$_, [table(NUMBER), table(NUMBER)]] }
+             map { [$_, [table(STRING, NUMBER), table(STRING, NUMBER)]] }
                dtypes(COMPLEX_DT, qw(
                    new
                    )
@@ -796,7 +792,7 @@ package Sidef::Optimizer {
         if ($ref eq 'HASH') {
             $obj = $self->optimize($obj);
         }
-        elsif ($ref eq "Sidef::Variable::Variable") {
+        elsif ($ref eq 'Sidef::Variable::Variable') {
             if ($obj->{type} eq 'var') {
                 ## ok
             }
@@ -809,13 +805,19 @@ package Sidef::Optimizer {
                 }
             }
         }
-        elsif ($ref eq 'Sidef::Variable::Static') {
+        elsif (
+               (
+                   $ref eq 'Sidef::Variable::Static'
+                or $ref eq 'Sidef::Variable::Const'
+                or $ref eq 'Sidef::Variable::Define'
+               )
+               and exists($obj->{expr})
+          ) {
             if ($addr{refaddr($obj)}++) {
                 ## ok
             }
             else {
-                my %code = $self->optimize($obj->{expr});
-                $obj->{expr} = \%code;
+                $obj->{expr} = {$self->optimize($obj->{expr})};
             }
         }
         elsif ($ref eq 'Sidef::Variable::Init') {
@@ -824,8 +826,7 @@ package Sidef::Optimizer {
             }
             else {
                 if (exists $obj->{args}) {
-                    my %code = $self->optimize($obj->{args});
-                    $obj->{args} = \%code;
+                    $obj->{args} = {$self->optimize($obj->{args})};
                 }
             }
         }
@@ -840,8 +841,16 @@ package Sidef::Optimizer {
                 ## ok
             }
             else {
-                my %code = $self->optimize($obj->{block}{code});
-                $obj->{block}{code} = \%code;
+                $obj->{block}{code} = {$self->optimize($obj->{block}{code})};
+            }
+        }
+        elsif ($ref eq 'Sidef::Types::Block::Try') {
+            if ($addr{refaddr($obj)}++) {
+                ## ok
+            }
+            else {
+                $obj->{try}{code}   = {$self->optimize($obj->{try}{code})};
+                $obj->{catch}{code} = {$self->optimize($obj->{catch}{code})} if defined($obj->{catch});
             }
         }
         elsif ($ref eq 'Sidef::Types::Block::BlockInit') {
@@ -849,8 +858,7 @@ package Sidef::Optimizer {
                 ## ok
             }
             else {
-                my %code = $self->optimize($obj->{code});
-                $obj->{code} = \%code;
+                $obj->{code} = {$self->optimize($obj->{code})};
             }
         }
         elsif ($ref eq 'Sidef::Types::Array::HCArray') {
@@ -867,60 +875,67 @@ package Sidef::Optimizer {
 
             # Has no expressions, so let's convert it into an Array
             #~ if (not $has_expr) {
-
             #~ #$obj = Sidef::Types::Array::Array->new(@{$obj});
             #~ bless $obj, 'Sidef::Types::Array::Array';
             #~ }
         }
         elsif ($ref eq 'Sidef::Types::Block::If') {
             foreach my $i (0 .. $#{$obj->{if}}) {
-                my %code = $self->optimize($obj->{if}[$i]{block}{code});
-                $obj->{if}[$i]{block}{code} = \%code;
-
-                my %expr = $self->optimize($obj->{if}[$i]{expr});
-                $obj->{if}[$i]{expr} = \%expr;
+                $obj->{if}[$i]{block}{code} = {$self->optimize($obj->{if}[$i]{block}{code})};
+                $obj->{if}[$i]{expr} = {$self->optimize($obj->{if}[$i]{expr})};
             }
             if (exists $obj->{else}) {
-                my %code = $self->optimize($obj->{else}{block}{code});
-                $obj->{else}{block}{code} = \%code;
+                $obj->{else}{block}{code} = {$self->optimize($obj->{else}{block}{code})};
             }
         }
         elsif ($ref eq 'Sidef::Types::Block::With') {
             foreach my $i (0 .. $#{$obj->{with}}) {
-                my %code = $self->optimize($obj->{with}[$i]{block}{code});
-                $obj->{with}[$i]{block}{code} = \%code;
-
-                my %expr = $self->optimize($obj->{with}[$i]{expr});
-                $obj->{with}[$i]{expr} = \%expr;
+                $obj->{with}[$i]{block}{code} = {$self->optimize($obj->{with}[$i]{block}{code})};
+                $obj->{with}[$i]{expr} = {$self->optimize($obj->{with}[$i]{expr})};
             }
             if (exists $obj->{else}) {
-                my %code = $self->optimize($obj->{else}{block}{code});
-                $obj->{else}{block}{code} = \%code;
+                $obj->{else}{block}{code} = {$self->optimize($obj->{else}{block}{code})};
             }
         }
-        elsif (   $ref eq 'Sidef::Types::Block::While'
-               or $ref eq 'Sidef::Types::Block::ForIn'
-               or $ref eq 'Sidef::Types::Block::ForEach') {
-            my %expr = $self->optimize($obj->{expr});
-            $obj->{expr} = \%expr;
+        elsif ($ref eq 'Sidef::Types::Block::ForIn') {
+            foreach my $loop (@{$obj->{loops}}) {
+                $loop->{expr} = {$self->optimize($loop->{expr})};
+            }
 
-            my %code = $self->optimize($obj->{block}{code});
-            $obj->{block}{code} = \%code;
+            $obj->{block}{code} = {$self->optimize($obj->{block}{code})};
+        }
+        elsif (   $ref eq 'Sidef::Types::Block::While'
+               or $ref eq 'Sidef::Types::Block::ForEach') {
+            $obj->{expr} = {$self->optimize($obj->{expr})};
+            $obj->{block}{code} = {$self->optimize($obj->{block}{code})};
         }
         elsif ($ref eq 'Sidef::Types::Block::CFor') {
             foreach my $i (0 .. $#{$obj->{expr}}) {
-                my %expr = $self->optimize($obj->{expr}[$i]);
-                $obj->{expr}[$i] = \%expr;
+                $obj->{expr}[$i] = {$self->optimize($obj->{expr}[$i])};
             }
-            my %code = $self->optimize($obj->{block}{code});
-            $obj->{block}{code} = \%code;
+            $obj->{block}{code} = {$self->optimize($obj->{block}{code})};
         }
         elsif ($ref eq 'Sidef::Variable::NamedParam') {
-            $obj->[1] = [map { {main => [$self->optimize_expr({self => $_})]} } @{$obj->[1]}];
+            $obj->{value} = [map { {main => [$self->optimize_expr({self => $_})]} } @{$obj->{value}}];
         }
         elsif ($ref eq 'Sidef::Meta::PrefixMethod') {
-            my %expr = $self->optimize($obj->{expr});
-            $obj->{expr} = \%expr;
+            $obj->{expr} = {$self->optimize($obj->{expr})};
+        }
+        elsif ($ref eq 'Sidef::Meta::Assert') {
+            $obj->{arg} = {$self->optimize($obj->{arg})};
+        }
+        elsif ($ref eq 'Sidef::Meta::Module') {
+            $obj->{block}{code} = {$self->optimize($obj->{block}{code})};
+        }
+        elsif ($ref eq 'Sidef::Meta::Included') {
+            foreach my $info (@{$obj->{included}}) {
+                $info->{ast} = {$self->optimize($info->{ast})};
+            }
+        }
+        elsif ($ref eq 'Sidef::Types::Bool::Ternary') {
+            $obj->{cond}  = {$self->optimize($obj->{cond})};
+            $obj->{true}  = {$self->optimize($obj->{true})};
+            $obj->{false} = {$self->optimize($obj->{false})};
         }
 
         if (not exists($expr->{ind}) and not exists($expr->{call})) {
@@ -979,9 +994,9 @@ package Sidef::Optimizer {
                     foreach my $j (0 .. $#{$call->{arg}}) {
                         my $arg = $call->{arg}[$j];
                         push @{$obj->{call}[$i]{arg}},
-                            ref($arg) eq 'HASH' ? do { my %arg = $self->optimize($arg); \%arg }
-                          : ref($arg) ? $self->optimize_expr({self => $arg})
-                          :             $arg;
+                            ref($arg) eq 'HASH' ? {$self->optimize($arg)}
+                          : ref($arg)           ? $self->optimize_expr({self => $arg})
+                          :                       $arg;
                     }
                 }
 
@@ -990,9 +1005,9 @@ package Sidef::Optimizer {
                     foreach my $j (0 .. $#{$call->{block}}) {
                         my $arg = $call->{block}[$j];
                         push @{$obj->{call}[$i]{block}},
-                            ref $arg eq 'HASH' ? do { my %arg = $self->optimize($arg); \%arg }
-                          : ref($arg) ? $self->optimize_expr({self => $arg})
-                          :             $arg;
+                            ref $arg eq 'HASH' ? {$self->optimize($arg)}
+                          : ref($arg)          ? $self->optimize_expr({self => $arg})
+                          :                      $arg;
                     }
                 }
 
@@ -1037,6 +1052,9 @@ package Sidef::Optimizer {
                                 }
 
                                 if ($ok) {
+                                    if (exists $dt_table{$ref_obj}) {
+                                        $obj->{self} = $dt_table{$ref_obj};
+                                    }
                                     $obj->{self} = $obj->{self}->$code(@args);
                                     $ref_obj     = ref($obj->{self});
                                     $optimized   = 1;
