@@ -1,19 +1,21 @@
 package Sidef::Types::Glob::File {
 
-    use 5.014;
+    use utf8;
+    use 5.016;
 
     use parent qw(
-      Sidef::Convert::Convert
       Sidef::Types::String::String
       );
 
+    require Encode;
+    require File::Spec;
+
     use Sidef::Types::Number::Number;
 
     sub new {
         my (undef, $file) = @_;
         if (@_ > 2) {
             shift(@_);
-            state $x = require File::Spec;
             $file = File::Spec->catfile(map { "$_" } @_);
         }
         elsif (ref($file) && ref($file) ne 'SCALAR') {
@@ -122,8 +124,8 @@ package Sidef::Types::Glob::File {
     sub readlink {
         ref($_[0]) || shift(@_);
         my ($self) = @_;
-        my $link = "$self";
-        my $class = (-d $link) ? 'Sidef::Types::Glob::Dir' : __PACKAGE__;
+        my $link   = "$self";
+        my $class  = (-d $link) ? 'Sidef::Types::Glob::Dir' : __PACKAGE__;
         $class->new(CORE::readlink($link));
     }
 
@@ -281,7 +283,6 @@ package Sidef::Types::Glob::File {
         ref($_[0]) || shift(@_);
         my ($self) = @_;
 
-        state $x = require File::Spec;
         File::Spec->file_name_is_absolute("$self")
           ? (Sidef::Types::Bool::Bool::TRUE)
           : (Sidef::Types::Bool::Bool::FALSE);
@@ -292,9 +293,11 @@ package Sidef::Types::Glob::File {
     sub abs_name {
         my $class = ref($_[0]) || shift(@_);
         my ($self, $base) = @_;
-
-        state $x = require File::Spec;
-        $class->new(File::Spec->rel2abs("$self", defined($base) ? "$base" : ()));
+        $class->new(
+                    Encode::decode_utf8(
+                          File::Spec->rel2abs(Encode::encode_utf8("$self"), defined($base) ? Encode::encode_utf8("$base") : ())
+                    )
+                   );
     }
 
     *abs     = \&abs_name;
@@ -304,9 +307,11 @@ package Sidef::Types::Glob::File {
     sub rel_name {
         my $class = ref($_[0]) || shift(@_);
         my ($self, $base) = @_;
-
-        state $x = require File::Spec;
-        $class->new(File::Spec->rel2abs("$self", defined($base) ? "$base" : ()));
+        $class->new(
+                    Encode::decode_utf8(
+                          File::Spec->rel2abs(Encode::encode_utf8("$self"), defined($base) ? Encode::encode_utf8("$base") : ())
+                    )
+                   );
     }
 
     *rel     = \&rel_name;
@@ -318,7 +323,7 @@ package Sidef::Types::Glob::File {
         my ($self) = @_;
 
         state $x = require Cwd;
-        $class->new(Cwd::abs_path("$self"));
+        $class->new(Encode::decode_utf8(Cwd::abs_path("$self")));
     }
 
     *realpath = \&abs_path;
@@ -367,12 +372,12 @@ package Sidef::Types::Glob::File {
         }
 
         truncate($fh, 0) || do {
-            warn "[WARN] Can't truncate file `$self': $!";
+            warn "[WARNING] Can't truncate file `$self`: $!";
             return undef;
         };
 
         seek($fh, 0, 0) || do {
-            warn "[WARN] Can't seek the begining of file `$self': $!";
+            warn "[WARNING] Can't seek the begining of file `$self`: $!";
             return undef;
         };
 
@@ -380,7 +385,7 @@ package Sidef::Types::Glob::File {
             local $, = q{};
             local $\ = q{};
             (print $fh @lines) || do {
-                warn "[WARN] Can't write to file `$self': $!";
+                warn "[WARNING] Can't write to file `$self`: $!";
                 return undef;
             };
             close $fh;
@@ -396,7 +401,7 @@ package Sidef::Types::Glob::File {
         $mode = defined($mode) ? "$mode" : 'utf8';
 
         open(my $fh, "<:$mode", "$self") || do {
-            warn "[WARN] Can't open file `$self' for writing: $!";
+            warn "[WARNING] Can't open file `$self` for reading: $!";
             return undef;
         };
 
@@ -411,12 +416,12 @@ package Sidef::Types::Glob::File {
         $mode = defined($mode) ? "$mode" : 'utf8';
 
         open(my $fh, ">:$mode", "$self") || do {
-            warn "[WARN] Can't open file `$self' for writing: $!";
+            warn "[WARNING] Can't open file `$self` for writing: $!";
             return undef;
         };
 
         (print $fh "$string") || do {
-            warn "[WARN] Can't write to file `$self': $!";
+            warn "[WARNING] Can't write to file `$self`: $!";
             return undef;
         };
 
@@ -432,12 +437,12 @@ package Sidef::Types::Glob::File {
         $mode = defined($mode) ? "$mode" : 'utf8';
 
         open(my $fh, ">>:$mode", "$self") || do {
-            warn "[WARN] Can't open file `$self' for appending: $!";
+            warn "[WARNING] Can't open file `$self` for appending: $!";
             return undef;
         };
 
         (print $fh "$string") || do {
-            warn "[WARN] Can't append to file `$self': $!";
+            warn "[WARNING] Can't append to file `$self`: $!";
             return undef;
         };
 
@@ -450,6 +455,7 @@ package Sidef::Types::Glob::File {
         ref($_[0]) || shift(@_);
         my ($self, $mode, $fh_ref, $err_ref) = @_;
 
+        $mode //= '<:utf8';
         $mode = "$mode" if (ref $mode);
 
         my $success = CORE::open(my $fh, $mode, ref($self) eq __PACKAGE__ ? $$self : "$self");
@@ -467,7 +473,7 @@ package Sidef::Types::Glob::File {
               };
         }
 
-        $success ? $fh_obj : ();
+        $success ? $fh_obj : undef;
     }
 
     sub touch {
@@ -618,14 +624,12 @@ package Sidef::Types::Glob::File {
     sub split {
         ref($_[0]) || shift(@_);
         my ($self) = @_;
-        state $x = require File::Spec;
         Sidef::Types::Array::Array->new([map { Sidef::Types::String::String->new($_) } File::Spec->splitdir("$self")]);
     }
 
     sub splitpath {
         ref($_[0]) || shift(@_);
         my ($self) = @_;
-        state $x = require File::Spec;
         Sidef::Types::Array::Array->new([map { Sidef::Types::String::String->new($_) } File::Spec->splitpath("$self")]);
     }
 
@@ -634,6 +638,12 @@ package Sidef::Types::Glob::File {
         Sidef::Types::String::String->new('File(' . ${Sidef::Types::String::String->new("$self")->dump} . ')');
     }
 
+    sub to_str {
+        my ($self) = @_;
+        Sidef::Types::String::String->new($$self);
+    }
+
+    *to_s = \&to_str;
 };
 
 1
