@@ -1,7 +1,8 @@
 package Sidef::Types::Glob::FileHandle {
 
     use utf8;
-    use 5.014;
+    use 5.016;
+
     use parent qw(
       Sidef::Object::Object
       );
@@ -28,6 +29,8 @@ package Sidef::Types::Glob::FileHandle {
         $_[0]{parent};
     }
 
+    *file = \&parent;
+
     sub is_on_tty {
         (-t $_[0]{fh}) ? (Sidef::Types::Bool::Bool::TRUE) : (Sidef::Types::Bool::Bool::FALSE);
     }
@@ -136,7 +139,7 @@ package Sidef::Types::Glob::FileHandle {
             do {
                 local $/;
                 CORE::readline($self->{fh});
-              }
+            }
         );
     }
 
@@ -192,21 +195,10 @@ package Sidef::Types::Glob::FileHandle {
 
         my @array;
 
-        if (ref($obj) eq 'Sidef::Types::Regex::Regex') {
-            my $re = $obj->{regex};
-            while (defined(my $line = CORE::readline($self->{fh}))) {
-                chomp($line);
-                if ($line =~ $re) {
-                    push @array, Sidef::Types::String::String->new($line);
-                }
-            }
-        }
-        else {
-            while (defined(my $line = CORE::readline($self->{fh}))) {
-                chomp($line);
-                my $string = Sidef::Types::String::String->new($line);
-                push @array, $string if $obj->run($string);
-            }
+        while (defined(my $line = CORE::readline($self->{fh}))) {
+            chomp($line);
+            my $string = Sidef::Types::String::String->new($line);
+            push @array, $string if $obj->run($string);
         }
 
         Sidef::Types::Array::Array->new(\@array);
@@ -225,6 +217,8 @@ package Sidef::Types::Glob::FileHandle {
         Sidef::Types::Array::Array->new(\@array);
     }
 
+    *collect = \&map;
+
     sub words {
         my ($self) = @_;
         Sidef::Types::Array::Array->new(
@@ -233,7 +227,7 @@ package Sidef::Types::Glob::FileHandle {
                  map    { Sidef::Types::String::String->new($_) }
                    grep { $_ ne '' }
                    split(' ', $_)
-               } CORE::readline($self->{fh})
+             } CORE::readline($self->{fh})
             ]
         );
     }
@@ -245,7 +239,7 @@ package Sidef::Types::Glob::FileHandle {
              map { Sidef::Types::String::String->new($_) } do {
                  local $/;
                  split(//, scalar CORE::readline($self->{fh}));
-               }
+             }
             ]
         );
     }
@@ -282,27 +276,26 @@ package Sidef::Types::Glob::FileHandle {
 
     sub tell {
         my ($self) = @_;
-        Sidef::Types::Number::Number->new(tell($self->{fh}));
+        Sidef::Types::Number::Number->new(CORE::tell($self->{fh}));
     }
 
     sub seek {
         my ($self, $pos, $whence) = @_;
-        seek($self->{fh}, $pos, $whence,)
+        CORE::seek($self->{fh}, $pos, $whence)
           ? (Sidef::Types::Bool::Bool::TRUE)
           : (Sidef::Types::Bool::Bool::FALSE);
     }
 
     sub sysseek {
         my ($self, $pos, $whence) = @_;
-
-        sysseek($self->{fh}, $pos, $whence,)
+        CORE::sysseek($self->{fh}, $pos, $whence)
           ? (Sidef::Types::Bool::Bool::TRUE)
           : (Sidef::Types::Bool::Bool::FALSE);
     }
 
     sub fileno {
         my ($self) = @_;
-        Sidef::Types::Number::Number->new(fileno($self->{fh}));
+        Sidef::Types::Number::Number->new(CORE::fileno($self->{fh}));
     }
 
     sub lock {
@@ -350,10 +343,6 @@ package Sidef::Types::Glob::FileHandle {
           : (Sidef::Types::Bool::Bool::FALSE);
     }
 
-    # File copy
-    *copy = \&Sidef::Types::Glob::File::copy;
-    *cp   = \&copy;
-
     sub read_to {
         my ($self, $var_ref) = @_;
 
@@ -376,12 +365,27 @@ package Sidef::Types::Glob::FileHandle {
         $self;
     }
 
+    sub copy {
+        my ($self, $fh) = @_;
+
+        if (ref($fh) ne __PACKAGE__) {
+            return;
+        }
+
+        state $x = require File::Copy;
+        File::Copy::copy($self->{fh}, $fh->{fh})
+          ? (Sidef::Types::Bool::Bool::TRUE)
+          : (Sidef::Types::Bool::Bool::FALSE);
+    }
+
+    *cp = \&copy;
+
     {
         no strict 'refs';
         *{__PACKAGE__ . '::' . '>>'} = \&read_to;
-        *{__PACKAGE__ . '::' . '»'} = \&read_to;
+        *{__PACKAGE__ . '::' . '»'}  = \&read_to;
         *{__PACKAGE__ . '::' . '<<'} = \&write_from;
-        *{__PACKAGE__ . '::' . '«'} = \&write_from;
+        *{__PACKAGE__ . '::' . '«'}  = \&write_from;
     }
 
 };
