@@ -1,6 +1,8 @@
 package Sidef::Types::Regex::Match {
 
-    use 5.014;
+    use utf8;
+    use 5.016;
+
     use overload
       q{bool} => \&get_value,
       q{""}   => sub { CORE::join(' ', @{$_[0]->{captures}}) },
@@ -18,21 +20,22 @@ package Sidef::Types::Regex::Match {
         my (undef, %hash) = @_;
 
         my @captures;
-        if ($hash{self}{global}) {
-            pos($hash{obj}) = $hash{self}{pos};
-            my $match = $hash{obj} =~ /$hash{self}{regex}/g;
+        if ($hash{regex}{global}) {
+
+            pos($hash{string}) = CORE::int($hash{regex}{pos});
+            my $match = $hash{string} =~ /$hash{regex}{regex}/g;
 
             if ($match) {
-                $hash{self}{pos} = pos($hash{obj});
+                $hash{regex}{pos} = pos($hash{string});
 
                 foreach my $i (1 .. $#{+}) {
-                    push @captures, substr($hash{obj}, $-[$i], $+[$i] - $-[$i]);
+                    push @captures, substr($hash{string}, $-[$i], $+[$i] - $-[$i]);
                 }
 
                 $hash{matched} = 1;
             }
             else {
-                $hash{self}{pos} = 0;
+                $hash{regex}{pos} = 0;
                 $hash{matched} = 0;
             }
 
@@ -41,13 +44,12 @@ package Sidef::Types::Regex::Match {
             }
         }
         else {
-            @captures =
-              defined($hash{pos})
-              ? (substr($hash{obj}, $hash{pos}) =~ $hash{self}{regex})
-              : ($hash{obj} =~ $hash{self}{regex});
+            $hash{pos} = CORE::int($hash{pos} // 0);
+            $hash{pos} = 0 if ($hash{pos} < 0);
+            @captures  = substr($hash{string}, $hash{pos}) =~ $hash{regex}{regex};
 
-            $hash{matched} = (@captures != 0);
-            $hash{match_pos} = $hash{matched} ? [$-[0] + ($hash{pos} // 0), $+[0] + ($hash{pos} // 0)] : [];
+            $hash{matched}   = (@captures != 0);
+            $hash{match_pos} = $hash{matched} ? [$-[0] + $hash{pos}, $+[0] + $hash{pos}] : [];
 
             foreach my $key (keys %+) {
                 $hash{named_captures}{$key} = $+{$key};
@@ -56,12 +58,6 @@ package Sidef::Types::Regex::Match {
 
         $hash{captures} = \@captures;
 
-        #if (defined $hash{parser}) {
-        #    while (my ($key, $value) = each %{$hash{parser}{regexp_vars}}) {
-        #        $value->set_value(Sidef::Types::String::String->new($captures[$key - 1]));
-        #    }
-        #}
-
         bless \%hash, __PACKAGE__;
     }
 
@@ -84,19 +80,23 @@ package Sidef::Types::Regex::Match {
                     [@{$self->{_cached_pos} //= [map { Sidef::Types::Number::Number->_set_uint($_) } @{$self->{match_pos}}]}]);
     }
 
+    *match_pos = \&pos;
+
     sub captures {
         my ($self) = @_;
         Sidef::Types::Array::Array->new(
                            [@{$self->{_cached_cap} //= [map { Sidef::Types::String::String->new($_) } @{$self->{captures}}]}]);
     }
 
-    *cap  = \&captures;
-    *caps = \&captures;
+    *cap      = \&captures;
+    *caps     = \&captures;
+    *to_a     = \&captures;
+    *to_array = \&captures;
 
     sub named_captures {
         my ($self) = @_;
 
-        my $hash = Sidef::Types::Hash::Hash->new();
+        my $hash = Sidef::Types::Hash::Hash->new;
         while (my ($key, $value) = each %{$self->{named_captures}}) {
             $hash->{$key} = Sidef::Types::String::String->new($value);
         }
@@ -111,15 +111,28 @@ package Sidef::Types::Regex::Match {
         Sidef::Types::String::String->new(CORE::join("$sep", @{$self->{captures}}));
     }
 
-    sub to_s {
+    sub string {
+        my ($self) = @_;
+        Sidef::Types::String::String->new($self->{string});
+    }
+
+    sub regex {
+        my ($self) = @_;
+        $self->{regex};
+    }
+
+    sub to_str {
         my ($self) = @_;
         Sidef::Types::String::String->new("$self");
     }
 
+    *to_s = \&to_str;
+
     sub dump {
         my ($self) = @_;
-        my $re = $self->{self}->dump;
-        Sidef::Types::String::String->new("(${Sidef::Types::String::String->new($self->{obj})->dump} =~ $re)");
+        my $re     = $self->{regex}->dump;
+        my $str    = Sidef::Types::String::String->new($self->{string})->dump;
+        Sidef::Types::String::String->new("($str =~ $re)");
     }
 
     {
